#ifndef _ViewCellBsp_H__
#define _ViewCellBsp_H__

class ViewCell;
class Plane3;
class Mesh;

//namespace GtpVisibilityPreprocessor {
  
class BSPInterior;

/**
	BSPNode abstract class serving for interior and leaf node implementation
*/
class BSPNode 
{
public:
	/** Determines whether this node is a leaf or not
	@return true if leaf
    */
    virtual bool IsLeaf() const = 0;
    
    /** Determines whether this node is a root
	@return true if root
    */
    virtual bool IsRoot() const;

	protected:

		void SplitPoly(Plane3 *plane, Mesh *viewcell);

    /// parent of this node
    BSPInterior *mParent;
  };

  /** BSP interior node implementation */
  class BSPInterior : public BSPNode 
  {    
    /** @return false since it is an interior node */
    bool IsLeaf() const;

  protected:
    /// Splitting plane corresponding to this node
    Plane3 mPlane;
    /// back node
    BSPNode *mBack;
    /// front node
    BSPNode *mFront;
  };
  
  
  /** BSP leaf node implementation */
  class BSPLeaf : public BSPNode 
  {
  public:
	BSPLeaf(ViewCell *viewCell = NULL);
    
	/** @return true since it is an interior node */
    bool IsLeaf() const;

  protected:
    /// polygonal representation of this viewcell
    /// if NULL this note does not correspond to feasible viewcell
    ViewCell *mViewCell;
  };
  
  /** Implementation of the ViewCell BSP tree */
  class BSPTree 
  {
  public:
    /** Constructor takes a pointer to the cell corresponding to the whole
	viewspace */
    BSPTree(ViewCell *cell);
        
  protected:
    /// Pointer to the root of the tree
    BSPNode *mRoot;
    /// Pointer to the root cell of the viewspace */
	ViewCell *mRootCell;
  };
  
//}; // GtpVisibilityPreprocessor

#endif
